@@ -1,7 +1,12 @@
-"""Classic games cards based on emojis.
+"""Classic playing cards module.
 
-This is based on emojis and cards are created at runtime.
+Many games use classic game cards.
+For that reason pygame_cards implements them using a simple api.
+
+
+The current implementation is based on emojis and graphics are created at runtime.
 """
+
 from __future__ import annotations
 from dataclasses import dataclass
 from enum import Enum
@@ -12,13 +17,16 @@ import sys
 import pygame
 from pygame_cards.abstract import AbstractCard
 from pygame_cards.abstract import AbstractCardGraphics
-from pygame_emojis import load_emoji, load_svg, find_code, _SVG_DIR
+from pygame_emojis import load_emoji, emoji_to_surface, find_code, _EMOJIS_DIR
 
 from pygame_cards.effects import outer_halo, Decay
 from pygame_cards.set import CardsSet
+from pygame_cards.defaults import DefaultCardsSet
 
 
 class Colors(Enum):
+    """Colors of the cards."""
+
     SPADE = "♠"
     HEART = "♥"
     DIAMOND = "♦"
@@ -26,26 +34,29 @@ class Colors(Enum):
 
 
 class Level(Enum):
+    """Levels of special cards."""
+
     JACK = "J"
     QUEEN = "Q"
     KING = "K"
     AS = "A"
 
 
+# Matching color strings
 RGBColor: dict[Colors, str] = {
     Colors.SPADE: "black",
     Colors.HEART: "red",
     Colors.DIAMOND: "red",
     Colors.CLUB: "black",
 }
-# Colors to specify an emoji version
+# Emoji color codes to specify an emoji version
 HEXEmojiColor: dict[Colors, str] = {
     Colors.SPADE: "1F3FB",
     Colors.HEART: "1F3FC",
     Colors.DIAMOND: "1F3FD",
     Colors.CLUB: "1F3FF",
 }
-
+# Emoji matching the levers
 LevelEmojis: dict[Level, str] = {
     Level.KING: "🤴",
     Level.QUEEN: "👸",
@@ -53,6 +64,11 @@ LevelEmojis: dict[Level, str] = {
 }
 
 
+class classproperty(property):
+    def __get__(self, obj, cls=None):
+        return self.fget(cls)
+
+
 @dataclass
 class EmojisFrenchSuits(AbstractCardGraphics):
     """A graphics showing the cards with emojis."""
@@ -72,17 +88,11 @@ class EmojisFrenchSuits(AbstractCardGraphics):
 
     @cached_property
     def symbols_rows(self):
-        return [
-            self.size[1] / 8 * (i + 1) - self.icon_size[0] / 2
-            for i in range(7)
-        ]
+        return [self.size[1] / 8 * (i + 1) - self.icon_size[0] / 2 for i in range(7)]
 
     @cached_property
     def symbols_cols(self):
-        return [
-            self.size[0] / 4 * (i + 1) - self.icon_size[1] / 2
-            for i in range(3)
-        ]
+        return [self.size[0] / 4 * (i + 1) - self.icon_size[1] / 2 for i in range(3)]
 
     @cached_property
     def top_label(self) -> pygame.Surface:
@@ -153,19 +163,16 @@ class EmojisFrenchSuits(AbstractCardGraphics):
         )
         # Add a face to the card
         if self.card.number in LevelEmojis:
-
             code_list = find_code(LevelEmojis[self.card.number])
             code = "-".join(code_list)
-            emojis_files = [f for f in _SVG_DIR.rglob(f"{code}*.svg")]
+            emojis_files = [f for f in _EMOJIS_DIR.rglob(f"{code}*")]
             good_color = [
-                e
-                for e in emojis_files
-                if HEXEmojiColor[self.card.color] in e.stem
+                e for e in emojis_files if HEXEmojiColor[self.card.color] in e.stem
             ]
             # Dimension for the face
             w = self.size[0] * 0.8
             h = self.size[1] * 0.4
-            face_surf = load_svg(good_color[0], size=(w, h))
+            face_surf = emoji_to_surface(good_color[0], size=(w, h))
             s.blit(
                 pygame.transform.flip(face_surf, False, True),
                 ((self.size[0] - w) / 2, (self.size[1] - h)),
@@ -190,9 +197,7 @@ class EmojisFrenchSuits(AbstractCardGraphics):
             flipped_icon = pygame.transform.flip(icon_s, False, True)
             r = self.symbols_rows
             c = self.symbols_cols
-            self.logger.debug(
-                f"{r=}, {c=}, {s.get_size()}, {icon_s.get_size()}"
-            )
+            self.logger.debug(f"{r=}, {c=}, {s.get_size()}, {icon_s.get_size()}")
             if self.card.number in [2, 3]:
                 s.blit(icon_s, (c[1], r[0]))
                 s.blit(flipped_icon, (c[1], r[-1]))
@@ -235,45 +240,71 @@ class EmojisFrenchSuits(AbstractCardGraphics):
 
 @dataclass(repr=False, eq=False)
 class NumberCard(AbstractCard):
-
     number: int | Level
     color: Colors
 
     graphics_type = EmojisFrenchSuits
 
+    def is_one_level_less_than(
+        self, other_card: NumberCard, as_equals_1: bool = False
+    ) -> bool:
+        """Return whether this card is one level less than the other_card.
+
+        :arg as_equals_1: Whether the As should be considered as 1 (worst of all)
+            or if it should be considered as the Best card
+        """
+        match other_card.number:
+            case Level.KING:
+                return self.number == Level.QUEEN
+            case Level.QUEEN:
+                return self.number == Level.JACK
+            case Level.JACK:
+                return self.number == 10
+            case Level.AS:
+                return False if as_equals_1 else self.number == Level.KING
+            case 2:
+                return self.number == Level.AS if as_equals_1 else False
+            case int():
+                if isinstance(self.number, Level):
+                    return False
+                return other_card.number == self.number + 1
+            case _:
+                raise NotImplementedError(f"{other_card.number = }")
+
 
 class CardSets:
     """Default card sets that can be used.
 
-    :attr n52: A 52 game cards.
-    :attr n36: A 36 game cards.
+    :param n52: A 52 game cards.
+    :param n36: A 36 game cards.
     """
 
-    @classmethod
-    @property
-    def n52(self) -> CardsSet[NumberCard]:
-        return CardsSet(
+    @classproperty
+    def n52(self) -> DefaultCardsSet[NumberCard]:
+        cardset = DefaultCardsSet(
             [
                 NumberCard(f"{n} of {c.value}", n, c)
                 for c in Colors
                 for n in [i for i in range(2, 11)] + [l for l in Level]
             ]
         )
+        cardset.name = "n52"
+        return cardset
 
-    @classmethod
-    @property
+    @classproperty
     def n36(self) -> CardsSet[NumberCard]:
-        return CardsSet(
+        cardset = DefaultCardsSet(
             [
                 NumberCard(f"{n} of {c.value}", n, c)
                 for c in Colors
                 for n in [i for i in range(6, 11)] + [l for l in Level]
             ]
         )
+        cardset.name = "n36"
+        return cardset
 
 
 if __name__ == "__main__":
-
     # This will visualize the cards
 
     pygame.init()
@@ -288,13 +319,9 @@ if __name__ == "__main__":
     print(set)
 
     # Add event to show the first card
-    pygame.event.post(
-        pygame.event.Event(pygame.KEYDOWN, {"key": pygame.K_LEFT})
-    )
+    pygame.event.post(pygame.event.Event(pygame.KEYDOWN, {"key": pygame.K_LEFT}))
     while 1:
-
         for event in pygame.event.get():
-
             if event.type == pygame.QUIT:
                 sys.exit()
 
