@@ -1,104 +1,31 @@
-from abc import abstractmethod, abstractproperty
-from dataclasses import dataclass
 from enum import auto
 from functools import cached_property
 import logging
 from math import cos, sin, sqrt
 import math
-from time import sleep
+
 import pygame
+
 from pygame_cards.abstract import AbstractCard
 from pygame_cards.utils import AutoName
-from pygame_cards.abstract import AbstractGraphic
 from pygame_cards.effects import outer_halo
-from pygame_cards.set import CardsSet
+from pygame_cards.set import CardsSet, CardsetGraphic
+from pygame_cards import constants
 
 
 class CardOverlap(AutoName):
+    """How card overlap in the hand."""
+
     left = auto()
     right = auto()
 
 
-@dataclass
-class CardsetGraphic(AbstractGraphic):
-    """A base graphic fro any card holder.
-
-    :attr cardset: The set of card that is shown.
-    :attr size: A tuple with the size of the requested card in this graphic.
-    :attr card_size: A tuple with the size of the cards in this graphic.
-    :attr card_border_radius: The radius of the cards in this card set.
-    :attr max_cards: The max number of card that can be contained in this
-        graphic. If 0, this is unlimited.
-    """
-
-    cardset: CardsSet
-
-    # Defualt hand size
-    size: tuple[int, int] = (900, 240)
-    card_size: tuple[int, int] = (135, 200)
-    card_border_radius: int = 20
-
-    max_cards: int = 0
-
-    graphics_type: type | None = None
-
-    def __post_init__(self):
-
-        for card in self.cardset:
-            # Enforce the type
-            if self.graphics_type:
-                card.graphics_type = self.graphics_type
-            # Enforce the card size
-            card.graphics.size = self.card_size
-        self._raised_with_hovered_warning = False
-
-    @abstractproperty
-    def surface(self) -> pygame.Surface:
-        raise NotImplementedError(
-            f"property 'surface' in {type(self).__name__}"
-        )
-
-    @abstractmethod
-    def get_card_at(self, pos: tuple[int, int]) -> AbstractCard | None:
-        """Return the card at the given pixel position
-
-        :arg pos: The position inside the CardsetGraphic surface.
-        """
-        raise NotImplementedError(
-            f"'get_card_at' in Class {type(self).__name__}"
-        )
-
-    def remove_card(self, card: AbstractCard) -> None:
-        """Remove a card from the cardset."""
-
-        self.cardset.remove(card)
-        self.clear_cache()
-
-    def append_card(self, card: AbstractCard) -> None:
-        """Append a card to the cardset."""
-
-        self.cardset.append(card)
-        card.graphics.size = self.card_size
-        card.graphics.clear_cache()
-        self.clear_cache()
-
-    def with_hovered(self, card: AbstractCard | None) -> pygame.Surface:
-        """Show the hand with the card hovered."""
-        if not self._raised_with_hovered_warning:
-            logging.warning(
-                f"Not implemented `with_hovered()` in {type(self).__name__}",
-            )
-            self._raised_with_hovered_warning = True
-        return self.surface
-
-
-@dataclass
 class BaseHand(CardsetGraphic):
     """A base class for a hand.
 
     Usually in a hand the cards are shown on next to the other.
 
-    :attr overlap_hide: The card to hide if to cards are one
+    :param overlap_hide: The card to hide if to cards are one
         over each other. By default the card overlap on the right,
         which is the standard in card games.
         This also implies that the cards are located at the opposite side
@@ -107,14 +34,20 @@ class BaseHand(CardsetGraphic):
 
     """
 
-    overlap_hide: CardOverlap = CardOverlap.right
+    def __init__(
+        self,
+        *args,
+        overlap_hide: CardOverlap = CardOverlap.right,
+        **kwargs,
+    ):
+        super().__init__(*args, **kwargs)
+        self.overlap_hide = overlap_hide
 
 
-@dataclass
 class AlignedHand(BaseHand):
     """A hand of card with all the cards aligned.
 
-    :attr offset_position: The offset of the position between the cards.
+    :param card_spacing: The offset proportion of the position between the cards.
 
         * 0, means the cards are directly next to each other.
         * Positive offset, means the card will be further
@@ -124,7 +57,14 @@ class AlignedHand(BaseHand):
 
     """
 
-    offset_position: float = 20
+    def __init__(
+        self,
+        *args,
+        card_spacing: float = constants.COLUMN_SPACING,
+        **kwargs,
+    ):
+        super().__init__(*args, **kwargs)
+        self.card_spacing = card_spacing
 
     @cached_property
     def surface(self) -> pygame.Surface:
@@ -158,30 +98,30 @@ class AlignedHand(BaseHand):
             spacing between the cards.
         """
         # calculate dimenstions required for the displayed surf
-        offset = self.offset_position
+        offset = self.card_spacing * self.card_size[0]
         total_x = (
-            len(self.cardset) * self.card_size[0]
-            + (len(self.cardset) - 1) * offset
+            len(self.cardset) * self.card_size[0] + (len(self.cardset) - 1) * offset
         )
 
         if total_x > self.size[0]:
-            self.logger.warning(
-                "Too many cards for hands size, rescaling will apply."
-            )
+            self.logger.warning("Too many cards for hands size, rescaling will apply.")
             offset = (self.size[0] - len(self.cardset) * self.card_size[0]) / (
                 len(self.cardset) - 1
             )
         x_positions = [
-            i * self.card_size[0] + (i + 1) * offset
-            for i in range(len(self.cardset))
+            i * self.card_size[0] + i * offset for i in range(len(self.cardset))
         ]
         # Revert the position in case of another overlap
         x_positions = [
-            x_pos
-            if self.overlap_hide == CardOverlap.right
-            else self.size[0] - self.card_size[0] - x_pos
+            (
+                x_pos
+                if self.overlap_hide == CardOverlap.right
+                else self.size[0] - self.card_size[0] - x_pos
+            )
             for x_pos in x_positions
         ]
+        self.logger.debug(f"{x_positions=}")
+
         return x_positions, offset
 
     def with_hovered(
@@ -195,9 +135,7 @@ class AlignedHand(BaseHand):
         x_pos = x_posistions[index]
 
         card.graphics.size = self.card_size
-        highlighted_surf = outer_halo(
-            card.graphics.surface, radius=radius, **kwargs
-        )
+        highlighted_surf = outer_halo(card.graphics.surface, radius=radius, **kwargs)
         # assume the center will be on it
         out_surf = pygame.Surface(self.size, pygame.SRCALPHA)
         highlighted_surf = pygame.transform.scale(
@@ -215,48 +153,52 @@ class AlignedHand(BaseHand):
         return out_surf
 
     def get_card_at(self, pos: tuple[int, int]) -> AbstractCard | None:
-        """Return the card at the given pixel position
-
-        :arg pos: The position inside the CardsHand surface.
-        """
         s = self.surface.get_size()
         if not (pos[0] < s[0] and pos[1] < s[1]):
-            self.logger.error(f"get_card_at:{pos=} not in {s}.")
+            self.logger.error(f"get_card_at({pos=}) not in {s}.")
             return None
 
         x_positions, offset = self.calculate_x_positions()
-        self.logger.debug(f"get_card_at:{x_positions=}, {self.overlap_hide=}")
+        self.logger.debug(f"get_card_at({pos=}): {x_positions=}, {self.overlap_hide=}")
 
         # TODO: make this work
         for card_index, x_pos in enumerate(x_positions):
             if pos[0] < x_pos or pos[0] > x_pos + self.card_size[0]:
                 self.logger.debug(
-                    f"get_card_at: {card_index=} is not Between the card boundaries"
+                    f"get_card_at({pos=}):: {card_index=} is not Between the card"
+                    " boundaries"
                 )
                 continue
             if (
                 self.overlap_hide == CardOverlap.right
                 # Check that is not under the next card
-                and pos[0] - x_pos > self.card_size[0] + self.offset_position
+                and pos[0] - x_pos
+                > self.card_size[0] + self.card_spacing * self.card_size[0]
             ) or (
                 self.overlap_hide == CardOverlap.left
-                and pos[0] - x_pos < self.offset_position
+                and pos[0] - x_pos < self.card_spacing * self.card_size[0]
             ):
                 self.logger.debug(
-                    f"get_card_at: {card_index=} is  going to be under the next card"
+                    f"get_card_at({pos=}):: {card_index=} is  going to be under the"
+                    " next card"
                 )
                 continue
             if x_pos - pos[0] > self.card_size[0] + offset:
-                self.logger.debug(f"get_card_at:{pos=} is in offset.")
+                self.logger.debug(f"get_card_at({pos=}): is in offset.")
                 continue  # Between two cards, in the offset
 
-            self.logger.debug(f"get_card_at:{pos=} found index {card_index}.")
+            self.logger.debug(f"get_card_at({pos=}): found index {card_index}.")
             return self.cardset[card_index]
 
         return None
 
+    def get_card_positions(self) -> dict[AbstractCard, tuple[int, int]]:
+        y = self.calculate_y_position()
+        xs, _ = self.calculate_x_positions()
+
+        return {card: (x, y) for card, x in zip(self.cardset, xs)}
+
 
-@dataclass
 class RoundedHand(BaseHand):
     """A hand of card with all the cards aligned on an arc of a circle.
 
@@ -264,7 +206,7 @@ class RoundedHand(BaseHand):
     the size of the hand.
 
 
-    :attr angle: The angle in which the cards are constrained
+    :param angle: The angle in which the cards are constrained
         (Unit: Degrees)
         If 0, the cards are all aligned.
         If not zero, the cards will be placed on an arc of a circle
@@ -272,9 +214,18 @@ class RoundedHand(BaseHand):
 
     """
 
-    angle: float = 90
-    # Change the defualt size
-    size: tuple[float, float] = (700, 400)
+    def __init__(
+        self,
+        *args,
+        angle: float = 90,
+        **kwargs,
+    ):
+        if not "size" in kwargs:
+            # Change the defualt size
+            kwargs["size"] = (700, 400)
+
+        super().__init__(*args, **kwargs)
+        self.angle = angle
 
     def _max_card_h(self) -> float:
         """Return the maximum height that a card can do in surface."""
@@ -296,17 +247,14 @@ class RoundedHand(BaseHand):
             self.logger.debug(f"{angle_step=}")
             # from the center, angle = 0, which is the central card and ref point.
             angles = [
-                -self.angle / 2 + i * angle_step
-                for i in range(len(self.cardset))
+                -self.angle / 2 + i * angle_step for i in range(len(self.cardset))
             ]
         # Radius of the circle around the cards (from center to card centers)
         # Trust me, I am an engineer
         card_diagonal = sqrt(self.card_size[0] ** 2 + self.card_size[1] ** 2)
         radius = min(
             # Constraint for width
-            (self.size[0] - card_diagonal)
-            / 2
-            / sin(math.radians(self.angle / 2)),
+            (self.size[0] - card_diagonal) / 2 / sin(math.radians(self.angle / 2)),
             # Constraint for heigth
             (self.size[1] - self.card_size[1] - card_diagonal / 2)
             / (1 - cos(math.radians(self.angle / 2))),
@@ -353,7 +301,6 @@ class RoundedHand(BaseHand):
         self.logger.debug(f"{card_positions=}")
 
         for card_surf, card_pos in zip(rotated_surfs, card_positions):
-
             surf.blit(
                 card_surf,
                 card_pos,
@@ -368,12 +315,7 @@ class RoundedHand(BaseHand):
 
     def get_card_at(self, pos: tuple[int, int]) -> AbstractCard | None:
         # We think pos1 from bottom instead of top
-        if (
-            pos[0] < 0
-            or pos[1] < 0
-            or pos[0] > self.size[0]
-            or pos[1] > self.size[1]
-        ):
+        if pos[0] < 0 or pos[1] < 0 or pos[0] > self.size[0] or pos[1] > self.size[1]:
             self.logger.debug(f"{pos=} ut of bound")
             return None
 
@@ -382,8 +324,7 @@ class RoundedHand(BaseHand):
         self.logger.debug(f"Center {self._center_pos=}")
 
         dist_to_center = sqrt(
-            (pos[0] - self._center_pos[0]) ** 2
-            + (pos[1] - self._center_pos[1]) ** 2
+            (pos[0] - self._center_pos[0]) ** 2 + (pos[1] - self._center_pos[1]) ** 2
         )
         self.logger.debug(f"{dist_to_center=}")
         if (
@@ -397,14 +338,8 @@ class RoundedHand(BaseHand):
             # Reverse because cards at the end are over the previous
             self.logger.debug(f"Check {card=} with {angle=}")
             card_center = (
-                (
-                    self._center_pos[0]
-                    + sin(math.radians(angle)) * self._radius
-                ),
-                (
-                    self._center_pos[1]
-                    + cos(math.radians(angle)) * self._radius
-                ),
+                (self._center_pos[0] + sin(math.radians(angle)) * self._radius),
+                (self._center_pos[1] + cos(math.radians(angle)) * self._radius),
             )
             dist_to_card_center = sqrt(
                 (pos[0] - card_center[0]) ** 2 + (pos[1] - card_center[1]) ** 2
@@ -412,11 +347,7 @@ class RoundedHand(BaseHand):
             # Cosine theorem to find angle (center, card, pos)
             # {\displaystyle a^{2}=c^{2}+b^{2}-2bc\cos \alpha }.
             a = math.acos(
-                (
-                    dist_to_center**2
-                    - dist_to_card_center**2
-                    - self._radius**2
-                )
+                (dist_to_center**2 - dist_to_card_center**2 - self._radius**2)
                 / (-2 * dist_to_card_center * self._radius)
             )
             # Distance to the line from the center to the center of the card
@@ -426,32 +357,236 @@ class RoundedHand(BaseHand):
                 return card
         return None
 
+    def get_card_positions(self) -> dict[AbstractCard, tuple[int, int]]:
+        # use aligned hand as a proxy
+        self.card_spacing = constants.COLUMN_SPACING
+        y = AlignedHand.calculate_y_position(self)
+        xs, _ = AlignedHand.calculate_x_positions(self)
 
-@dataclass
-class PlayingHand(BaseHand):
-    """A representation of the hand of the player.
+        self.logger.warning(
+            f"'get_card_positions' is not perfectly correct implemented yet"
+        )
 
-    This will show the cards how the player wants it.
+        return {card: (x, y) for card, x in zip(self.cardset, xs)}
+
+
+class Pile(CardsetGraphic):
+    """A pile has only its last/s card/s that can be selected.
+
+    :param rel_offset: The offset between cards. Relative to the card size.
 
-    Currently only works with size and card_size being specified.
-    TODO: optimize by caching the variables.
     """
 
-    @abstractmethod
-    def hovered(self, card: AbstractCard) -> pygame.Surface:
-        """Show the hand with the card hovered."""
+    def __init__(
+        self,
+        *args,
+        rel_offset: float = 0.2,
+        **kwargs,
+    ):
+        super().__init__(*args, **kwargs)
+        self.rel_offset = rel_offset
 
-    @abstractmethod
-    def with_going_to_play(self, card: AbstractCard) -> pygame.Surface:
-        """Show the hand with the card that is going to be played.
+    def _get_card_index_at(self, pos: tuple[int, int]) -> int | None:
+        """Return the index of the card located at the current position."""
+        raise NotImplementedError(f"Must implement in {type(self).__name__}")
 
-        This can be used for asking a confimation to the user,
-        or for aking a way to play the card.
+    def get_card_at(self, pos: tuple[int, int]) -> AbstractCard | None:
+        idx = self._get_card_index_at(pos)
+
+        return None if idx is None else self.cardset[idx]
+
+    def get_cards_at(self, pos: tuple[int, int]) -> CardsSet | None:
+        """Return the card set at the given pixel position
+
+        :arg pos: The position inside the CardsetGraphic surface.
         """
+        idx = self._get_card_index_at(pos)
+
+        return None if idx is None else self.cardset[idx:]
+
+
+class VerticalPileGraphic(Pile):
+    """Show a column in cards aligned.
+
+    Cards are shown from first one to the last one on the bottom.
+
+    """
+
+    def __init__(
+        self,
+        *args,
+        **kwargs,
+    ):
+        if not "size" in kwargs:
+            # Change the defualt size
+            kwargs["size"] = (175, 400)
+
+        super().__init__(*args, **kwargs)
+
+    def clear_cache(self) -> None:
+        # Remove the surface cache_property if exists
+        self.__dict__.pop("y_positions", None)
+        super().clear_cache()
+
+    @property
+    def size(self) -> tuple[int, int]:
+        return super().size
+
+    @size.setter
+    def size(self, size: tuple[int, int]) -> None:
+        super(VerticalPileGraphic, VerticalPileGraphic).size.__set__(self, size)
+        # Cards are too large for the new pile size
+        if self.card_size[0] > self.size[0]:
+            width = self.size[0]
+            # Keep the same ratio
+            height = width / self.card_size[0] * self.card_size[1]
+
+            self.card_size = (width, height)
+            self.logger.debug(f"setting card size to {self.card_size=}")
+
+    @cached_property
+    def surface(self) -> pygame.Surface:
+        # Create the surface
+        surf = pygame.Surface(self.size, pygame.SRCALPHA)
+
+        # Calculate how we position the cards
+        n_cards = len(self.cardset)
+        if n_cards == 0:
+            return surf
+
+        self.logger.debug(f"{self.size=}, {self.card_size=}")
+
+        x_position = (self.size[0] - self.card_size[0]) / 2
+
+        # Add the cards on the surface
+        surf.blits(
+            [
+                (card.graphics.surface, (x_position, y))
+                for card, y in zip(self.cardset, self.y_positions)
+            ],
+        )
+
+        return surf
+
+    @cached_property
+    def y_positions(self) -> list[int]:
+        n_cards = len(self.cardset)
+        max_offset = self.rel_offset * self.card_size[1]
+
+        expected_h = n_cards * max_offset + self.card_size[1]
+        h_space = (
+            (self.size[1] - self.card_size[1]) / n_cards
+            if expected_h > self.size[1]
+            else max_offset
+        )
+        return [i * h_space for i in range(n_cards)]
+
+    def _get_card_index_at(self, pos: tuple[int, int]) -> int | None:
+        """Return the index of the card located at the current position."""
+        if not self.cardset:
+            # No cards case
+            return None
+        # Iterate over the cards and position to find the correct card
+
+        for card_idx in range(len(self.cardset)):
+            if pos[1] < self.y_positions[card_idx]:
+                return card_idx - 1
+        if pos[1] < self.y_positions[-1] + self.card_size[1]:
+            # Last card is on top
+            return len(self.cardset) - 1
+        return None
+
+    def get_card_positions(self) -> dict[AbstractCard, tuple[int, int]]:
+        x_position = (self.size[0] - self.card_size[0]) / 2
+
+        return {
+            card: (x_position, y) for card, y in zip(self.cardset, self.y_positions)
+        }
+
+
+class HorizontalPileGraphic(Pile):
+    """Show a cards horizontally aligned.
+
+    Cards are shown from first one to the last one on the bottom.
+
+    """
+
+    def __init__(
+        self,
+        *args,
+        **kwargs,
+    ):
+        if not "size" in kwargs:
+            # Change the defualt size
+            kwargs["size"] = (350, 240)
+
+        super().__init__(*args, **kwargs)
+
+    def clear_cache(self) -> None:
+        # Remove the surface cache_property if exists
+        self.__dict__.pop("x_positions", None)
+        super().clear_cache()
+
+    @cached_property
+    def surface(self) -> pygame.Surface:
+        # Create the surface
+        surf = pygame.Surface(self.size, pygame.SRCALPHA)
+
+        # Calculate how we position the cards
+        n_cards = len(self.cardset)
+        if n_cards == 0:
+            return surf
+
+        y_position = (self.size[1] - self.card_size[1]) / 2
+
+        # Add the cards on the surface
+        surf.blits(
+            [
+                (card.graphics.surface, (x, y_position))
+                for card, x in zip(self.cardset, self.x_positions)
+            ],
+        )
+
+        return surf
+
+    @cached_property
+    def x_positions(self) -> list[int]:
+        n_cards = len(self.cardset)
+        # Get the offset between cards in pixels
+        offset = self.rel_offset * self.card_size[0]
+
+        expected_h = n_cards * offset + self.card_size[0]
+        h_space = (
+            (self.size[0] - self.card_size[0]) / n_cards
+            if expected_h > self.size[0]
+            else offset
+        )
+        return [i * h_space for i in range(n_cards)]
+
+    def _get_card_index_at(self, pos: tuple[int, int]) -> int | None:
+        """Return the index of the card located at the current position."""
+        if not self.cardset:
+            # No cards case
+            return None
+        # Iterate over the cards and position to find the correct card
+
+        for card_idx in range(len(self.cardset)):
+            if pos[0] < self.x_positions[card_idx]:
+                return card_idx - 1
+        if pos[0] < self.x_positions[-1] + self.card_size[0]:
+            # Last card is on top
+            return len(self.cardset) - 1
+        return None
+
+    def get_card_positions(self) -> dict[AbstractCard, tuple[int, int]]:
+        y_position = (self.size[1] - self.card_size[1]) / 2
+
+        return {
+            card: (x, y_position) for card, x in zip(self.cardset, self.x_positions)
+        }
 
 
 if __name__ == "__main__":
-
     # This will visualize the cards
 
     import sys
@@ -462,21 +597,21 @@ if __name__ == "__main__":
 
     pygame.init()
 
-    size = width, height = 1500, 700
+    size = width, height = 1500, 1200
 
     screen = pygame.display.set_mode(size)
 
     card_set = CardsSet(ClassicCardSet.n52[:4])
 
     graphics_aligned = AlignedHand(card_set)
-    graphics_aligned_overlap = AlignedHand(card_set, offset_position=-29)
+    graphics_aligned_overlap = AlignedHand(card_set, card_spacing=-0.15)
     graphics_rounded = RoundedHand(card_set + card_set + card_set)
     graphics_rounded2 = RoundedHand(card_set + card_set)
 
-    graphics_aligned.logger.setLevel(logging.DEBUG)
+    # graphics_aligned.logger.setLevel(logging.DEBUG)
     graphics_aligned_overlap.logger.setLevel(logging.DEBUG)
-    graphics_rounded.logger.setLevel(logging.DEBUG)
-    graphics_rounded2.logger.setLevel(logging.DEBUG)
+    # graphics_rounded.logger.setLevel(logging.DEBUG)
+    # graphics_rounded2.logger.setLevel(logging.DEBUG)
 
     screen.blit(graphics_aligned.surface, (200, 0))
     screen.blit(graphics_aligned_overlap.surface, (200, 200))
@@ -485,16 +620,18 @@ if __name__ == "__main__":
 
     pygame.display.flip()
 
+    clock = pygame.time.Clock()
+    fps = 4
+
     while 1:
         pos = pygame.mouse.get_pos()
         hoverd_card = graphics_aligned.get_card_at((pos[0] - 200, pos[1] - 0))
         screen.blit(
-            graphics_aligned.with_hovered(hoverd_card, fill_inside=False),
+            graphics_aligned.with_hovered(hoverd_card, fill_inside=True),
             (200, 0),
         )
         pygame.display.update()
-        sleep(1)
+        clock.tick(fps)
         for event in pygame.event.get():
-
             if event.type == pygame.QUIT:
                 sys.exit()
